import json
import logging
import requests

logging.getLogger("urllib3").setLevel(logging.WARNING)

class Api:
    def __init__(self, network):
        if not network in ['goerli', 'mainnet']:
            raise Exception("--network can only be mainnet or goerli")
        self.network = network
        subdomain = "api" if network == "mainnet" else "api-testnet"
        self.api_url = f"https://{subdomain}.starkscan.co/api"
      
    def get_scanner_link(self, class_hash):
        domain = "starkscan.co" if self.network == "mainnet" else "testnet.starkscan.co"
        return f"https://{domain}/class/{class_hash}#code"

    def create_job(self, data):
        headers = {'Content-type': 'application/json'}
        endpoint = f"{self.api_url}/verify_class"
        res = requests.post(endpoint, json=data, headers=headers)

        if res.status_code == 200:
            response = json.loads(res.text)
            return response['job_id']
        elif res.status_code == 400:
            logging.error(res.text)
        else:
            logging.error(f"Got error {res.status_code}")
            logging.error(res.text)

    def get_job_status(self, job_id):
        res = requests.get(f"{self.api_url}/verify_class_job_status/{job_id}")
        response = json.loads(res.text)
        return response['status'], response
        
    def is_hash_verifiable(self, class_hash):
        res = requests.get(f"{self.api_url}/hash/{class_hash}")
        if res.status_code == 200:
            if json.loads(res.text)['is_verified']:
                logging.info("")
                logging.info(f"✨ Contract is already verified:")
                logging.info("")
                logging.info(f"       {self.get_scanner_link(class_hash)}")
                logging.info("")
            else:
                return True
        elif res.status_code == 404 and "could not find hash" in res.text:
            logging.error(f"❌ Could not find any contract with hash {class_hash}")
            logging.error(f"🤔 Are you sure you deployed to {self.network}?")
        else:
            logging.error(f"Got error {res.status_code}")
            logging.error(res.text)

        return False



# First, import click dependency
import time
import click
import logging
from os.path import basename, splitext
from nile.common import get_hash
from nile_verifier.api import Api
from yaspin import yaspin
from yaspin.spinners import Spinners

@click.command()
@click.argument("main_file", nargs=1)
@click.option("--network", nargs=1, required=True)
@click.option("--compiler_version", nargs=1, default="0.10.0")
def verify(main_file, network, compiler_version):
    """
    Command for automatically verify the sourcecode of a contract on starkscan.co.
    """
    api = Api(network)
    contract_name = get_contract_name(main_file)
    class_hash = hex(get_hash(contract_name))

    if api.is_hash_verifiable(class_hash):
        logging.info(f"🔎  Verifying {contract_name} on {network}...")
        job_id = api.create_job({
            "main_file_path": basename(main_file),
            "class_hash": class_hash,
            "name": contract_name,
            "compiler_version": compiler_version,
            "is_account_contract": check_is_account(main_file),
            "files": get_files(main_file),
        })

        status = 'PENDING'
        with yaspin(Spinners.earth, text="Waiting for verification result") as sp:
            while status == 'PENDING':
                time.sleep(1)
                status, response = api.get_job_status(job_id)

        if status == 'FAILED':
            logging.error("💥  Verification failed:")
            logging.error(response['error_message'])
        else:
            scanner_url = api.get_scanner_link(class_hash)
            logging.info(f"✅  Success! {scanner_url}")


def check_is_account(main_file):
    # to do: improve detection
    contract_name = get_contract_name(main_file)
    return contract_name.endswith("Account")

def get_files(main_file):
    # to do: support multifile
    contract_paths = [main_file]

    files = {}
    for contract_path in contract_paths:
        contract_filename = basename(contract_path)
        with open(contract_path) as f:
            files[contract_filename] = f.read()

    return files

def get_contract_name(path):
    return splitext(basename(path))[0]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nile_verifier']

package_data = \
{'': ['*']}

install_requires = \
['cairo-nile>=0.9.0,<0.10.0',
 'click>=8.1.3,<9.0.0',
 'requests>=2.28.1,<3.0.0',
 'yaspin>=2.2.0,<3.0.0']

entry_points = \
{'nile_plugins': ['verify = nile_verifier.main.verify']}

setup_kwargs = {
    'name': 'nile-verifier',
    'version': '0.1.2',
    'description': 'Nile plugin to verify smart contracts on starkscan.co',
    'long_description': '# ⛵️✅ Nile verifier plugin\n\nPlugin for [Nile](https://github.com/OpenZeppelin/nile) to verify contracts on [starkscan.co](https://starkscan.co).\n\n## Installation\n\n```\npip install nile-verifier\n```\n\n## Usage\n\n```\nnile verify CONTRACT_PATH --network NETWORK\n```\n\nFor example:\n```\n$ nile verify contracts/uwu.cairo --network goerli\n🔎 Verifying uwu on goerli...\n✅ Success! https://testnet.starkscan.co/class/0x226718449b40fa299d718eb50f72af707f2210e540e11a830c2ad72a235d5e0#code\n```\n\nNote that the contract has to be deployed, or the verification will fail\n```\n$ nile verify contracts/uwu.cairo --network goerli\n❌ Could not find any contract with hash 0x226718449b40fa299d718eb50f72af707f2210e540e11a830c2ad72a235d5e0\n🤔 Are you sure you deployed to goerli?\n```\n\n## License\n\nMIT.\n\n',
    'author': 'Martín Triay',
    'author_email': 'martriay@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/martriay/nile-verifier-plugin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
